package	jp.co.sds_corp.fxutil;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

public class ForexCurve
{
	CcyPair		pair;
	Date		spotDate;
	ForexRate	spotRate;
	HashMap<String,ForexRate>	mapTerm = new HashMap<String,ForexRate>();
	TreeMap<Date,ForexRate>	mapDate = new TreeMap<Date,ForexRate>();
	IRateLoader	loader;
	MasterCache	master;
	CcyPair[]	cross = new CcyPair[2];

	public ForexCurve( CcyPair pair ) {
		this.pair = pair;
		master = MasterCache.getInstance();
		if ( pair.isCompound()) {
			cross[0] = master.getCcyPair( pair.getCcy1(), pair.getCross() );
			cross[1] = master.getCcyPair( pair.getCcy2(), pair.getCross() );
		}
		spotRate = new ForexRate( pair, "SP" );
		mapTerm.put( "SP", spotRate );
	}

	public CcyPair[] getCross() {
		if ( !pair.isCompound() ) {
			return null;
		}
		return cross;
	}

	public ForexRate addTerm( String term ) {
		ForexRate rate;
		if ( term.equals( "SP" )) {
			rate = spotRate;
		} else {
			rate = new ForexRate( pair, term );
		}
		mapTerm.put( term, rate );
		// @@@ 自動算出？
		// mapDate.put( date, rate );
		return rate;
	}
	public ForexRate addTerm( String term, Date date ) {
		ForexRate rate;
		if ( term.equals( "SP" )) {
			rate = spotRate;
			setSpotDate( date );
		} else {
			rate = new ForexRate( pair, term, date );
		}
		mapTerm.put( term, rate );
		mapDate.put( date, rate );
		return rate;
	}
	public ForexRate addTerm( Date date ) {
		ForexRate rate = new ForexRate( pair, date );
		mapDate.put( date, rate );
		return rate;
	}

	public void clearTerm() {
		mapTerm.clear();
		mapDate.clear();
		spotDate = null;
	}

	public void setSpotDate( Date date ) {
		if ( spotDate != null ) {
			mapDate.remove( spotDate );
		}
		spotDate = date;
		spotRate.setValueDate( date );
		mapDate.put( spotDate, spotRate );
	}
	public Date getSpotDate() {
		return spotDate;
	}

	public void setSpotRate( BigDecimal bid, BigDecimal ask ) {
		spotRate.setBid( bid );
		spotRate.setAsk( ask );
		spotRate.updateTime();
	}
	public ForexRate getSpotRate() throws PricingException {
		ForexRate ret = spotRate;
		if ( pair.isCompound()) {
			if ( pair.getProperty( "Adjust1" ) != null ||
				pair.getProperty( "Adjust2" ) != null )
			{
				ret = calcSpotAdjusted();
			} else {
				ret = calcCross( spotDate );
			}
		}
		return ret;
	}

	public ForexRate getTerm( String term ) throws PricingException {
		if ( pair.isCompound()) {
			// term 日付の取得
			// return getRate( date );
		}
		ForexRate ret = (ForexRate )mapTerm.get( term );
		return ret;
	}

	public void setRateLoader( IRateLoader var ) {
		this.loader = var;
	}

	public ForexRate getRate( Date date ) throws PricingException {
		if ( pair.isCompound()) {
			return calcCross( date );
		}
		ForexRate ret = new ForexRate( pair, date );
		Date from = null, to = null;
		Iterator<Date> itr = mapDate.keySet().iterator();
		while ( itr.hasNext()) {
			Date d = (Date )itr.next();
			ForexRate term = (ForexRate )mapDate.get( d );
			int cmp = d.compareTo( date );
			if ( cmp == 0 ) {
				ret.setTerm( term.getTerm() );
				ret.setAsk( term.getAsk() );
				ret.setBid( term.getBid() );
				break;
			} else if ( cmp > 0 ) {
				if ( from == null ) {
					// 最小term以前
					// @@@
				} else {
					to = d;
					// from - to で計算
					// @@@
				}
			} else {
				from = d;
			}
		}
		if ( from != null && to == null ) {
			// 最大term以上
			// @@@
		}
		return ret;
	}

	protected ForexRate calcCross( Date date ) throws PricingException {
		if ( loader == null ) {
			return null;
		}
		ForexRate ret = null;
		// 
		return ret;
	}

	protected ForexRate calcSpotAdjusted() throws PricingException {
		ForexRate ret = null;
		String var;
		var = pair.getProperty( "Adjust1" );
		BigDecimal adj1 = new BigDecimal( var );
		var = pair.getProperty( "Adjust2" );
		BigDecimal adj2 = new BigDecimal( var );
		return ret;
	}

	public Map<String,ForexRate> mapTerm() {
		return mapTerm;
	}

	public String toString() {
		String ret = "ForexCurve[" + pair + "] " + mapTerm;
		return ret;
	}

}
